CSM class

Constructors

CSM(CSMData data)

Properties

breaks ↔ List<double>
getter/setter pair
data ↔ CSMData
getter/setter pair
fade ↔ bool
getter/setter pair
frustums ↔ List<CSMFrustum>
getter/setter pair
hashCode → int
The hash code for this object.
no setterinherited
lights ↔ List<Light>
getter/setter pair
mainFrustum ↔ CSMFrustum
getter/setter pair
runtimeType → Type
A representation of the runtime type of the object.
no setterinherited
shaders ↔ Map
getter/setter pair

Methods

createLights() → void
dispose() → void
getBreaks() → void
getExtendedBreaks(List<Vector2> target) → void
initCascades() → void
injectInclude() → void
logarithmicSplit(int amount, double near, double far, List<double> target) → void
noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
practicalSplit(int amount, double near, double far, dynamic lambda, List<double> target) → void
remove() → void
setupMaterial(Material material) → void
toString() → String
A string representation of this object.
inherited
uniformSplit(int amount, double near, double far, List<double> target) → void
update() → void
updateFrustums() → void
updateShadowBounds() → void
updateUniforms() → void

Operators

operator ==(Object other) → bool
The equality operator.
inherited