WaterOptions class

Constructors

WaterOptions({int color = 0xffffff, int textureWidth = 512, int textureHeight = 512, double clipBias = 0.0, Map<String, dynamic>? shader, double scale = 1, double flowSpeed = 0.03, double reflectivity = 0.02, Vector2? flowDirection, dynamic flowMap, FutureOr<Texture?>? normalMap0, FutureOr<Texture?>? normalMap1})
WaterOptions.fromMap(Map<String, dynamic> options)

Properties

clipBias ↔ double
getter/setter pair
color ↔ int
getter/setter pair
flowDirection ↔ Vector2
getter/setter pair
flowMap ↔ dynamic
getter/setter pair
flowSpeed ↔ double
getter/setter pair
hashCode → int
The hash code for this object.
no setterinherited
normalMap0 ↔ FutureOr<Texture?>
getter/setter pair
normalMap1 ↔ FutureOr<Texture?>
getter/setter pair
reflectivity ↔ double
getter/setter pair
runtimeType → Type
A representation of the runtime type of the object.
no setterinherited
scale ↔ double
getter/setter pair
shader ↔ Map<String, dynamic>
getter/setter pair
textureHeight ↔ int
getter/setter pair
textureWidth ↔ int
getter/setter pair

Methods

noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
toString() → String
A string representation of this object.
inherited

Operators

operator ==(Object other) → bool
The equality operator.
inherited