AtmosphereParameters class
Constructors
- AtmosphereParameters({int particles = 4000, int minParticleSize = 50, int maxParticleSize = 100, double radius = 0.0, double thickness = 1.5, double density = 0, double opacity = 0.35, double scale = 8, Color? color, double speed = 0.03, Vector3? lightDirection})
-
AtmosphereParameters.fromMap([Map<
String, dynamic> ? map])
Properties
- color ↔ Color
-
getter/setter pair
- density ↔ double
-
getter/setter pair
- hashCode → int
-
The hash code for this object.
no setterinherited
-
json
→ Map<
String, dynamic> -
no setter
- lightDirection ↔ Vector3
-
getter/setter pair
- maxParticleSize ↔ int
-
getter/setter pair
- minParticleSize ↔ int
-
getter/setter pair
- opacity ↔ double
-
getter/setter pair
- particles ↔ int
-
getter/setter pair
- radius ↔ double
-
getter/setter pair
- runtimeType → Type
-
A representation of the runtime type of the object.
no setterinherited
- scale ↔ double
-
getter/setter pair
- speed ↔ double
-
getter/setter pair
- thickness ↔ double
-
getter/setter pair
-
uniforms
→ Map<
String, dynamic> -
no setter
Methods
-
noSuchMethod(
Invocation invocation) → dynamic -
Invoked when a nonexistent method or property is accessed.
inherited
-
setProperty(
String key, dynamic value) → void -
toString(
) → String -
A string representation of this object.
inherited
Operators
-
operator ==(
Object other) → bool -
The equality operator.
inherited
-
operator [](
dynamic key) → dynamic -
operator []=(
String key, dynamic value) → void